@@ -1,5 +1,5 @@
  /* sane - Scanner Access Now Easy.
-   Copyright (C) 2003 Johannes Hub (JohannesHub@foni.net)
+   Copyright (C) 2003 Johannes Hub (JohannesHub@t-online.de)
 
    This file was initially copied from the hp3300 backend.
    This file is part of the SANE package.
@@ -45,16 +45,23 @@
     such as HP3500C, 3530C, and HP ScanJet 3570C.
     Parts of this source were inspired by other backends.
 
-    History:
+		History:
 
-    Version 0.17g 18.04.04 10.alpha, little fixes
-    Version 0.17f 12.04.04 10.alpha, little fixes
-    Version 0.17e 07.04.04 10.alpha, little fixes
-    Version 0.17d 05.04.04 10.alpha, little fixes
-    Version 0.17c 31.03.04 10.alpha, little fixes
-    Version 0.17b 30.03.04 10.alpha, little fixes and libusb implemented
-    Version 0.17a 22.03.04 10.alpha, little fixes
-    Version 0.17  09.03.04 9. alpha, HP3500 included
+		Version 0.18  21.11.04 13.alpha,
+				- source sorted,
+				- now only SANEI_USB_SUPPORT for a better overview(xfermodules removed)
+				- read and verify the MainBoardID 
+		Version 0.17p 02.11.04 12.alpha, source sorted, little fixes, SANEI_USB_SUPPORT implemented
+		Version 0.17p 02.11.04 12.alpha, source sourted, little fixes, SANEI_USB_SUPPORT implemented
+		Version 0.17b 30.03.04 10.alpha, little fixes and libusb implemented
+		Version 0.17  09.03.04 9. alpha, HP3500 included
+		Version 0.16  06.02.04 8. alpha, wait counting on LCD
+		Version 0.15a 29.01.04 7. alpha, CCD switch moved to config file
+		Version 0.15  11.01.04 6. alpha, a second CCD implemented
+		Version 0.13a 21.11.04 4. alpha, an little fix included
+		Version 0.12  22.10.03 third alpha, Backend name changed to HP_RTS88xx
+		Version 0.11  30.08.03 second alpha
+		Version 0.10  19.07.03 first alpha
 */
 
 /*
@@ -62,8 +69,15 @@
 */
 
 #include <stdio.h>  /* fopen, fread, fwrite, fclose etc */
+#include <stdarg.h> /* va_list for vfprintf */
+#include <string.h> /* memcpy, memset */
+#include <unistd.h> /* unlink */
+#include <stdlib.h> /* malloc, free */
+#include <math.h>   /* exp, pow */
+#include <ctype.h>
 
 #include "hp_rts_xfer.h"
+#include "hp_rts_44x0c.h"
 #include "hp_rts_35x0c.h"
 
 static SANE_Byte sram_access_method = 0;
@@ -381,6 +395,7 @@ Hp35x0c_turn_on_lamp( SANE_Int iHandle )
 
 /****************************************************************************/
 /****************************************************************************/
+#if 0
 static SANE_Int
 Hp35x0c_set_value_msbfirst(	SANE_Byte	*regs,
 			SANE_Int		firstreg,
@@ -395,7 +410,7 @@ Hp35x0c_set_value_msbfirst(	SANE_Byte	*regs,
 	}
 	return 0;
 }
-
+#endif
 /****************************************************************************/
 SANE_Int
 Hp35x0c_set_ccd_shift_clock_multiplier(	SANE_Byte	*regs,
@@ -678,8 +693,12 @@ Hp35x0c_set_merge_channels(			SANE_Byte *regs,
 /****************************************************************************/
 					SANE_Int	on)
 {
+	DBG( DBG_MSG , "start_scan: Hp35x0c_set_merge_channels\n");
 	regs[0x2f] &= ~0x14;
 	regs[0x2f] |= on ? 0x04 : 0x10;
+#ifdef DEBUG
+	Hp_rts_DumpBits(0x2f, regs[0x2f]);
+#endif
 	return 0;
 }
 
@@ -689,7 +708,11 @@ Hp35x0c_set_channel(				SANE_Byte *regs,
 /****************************************************************************/
 					SANE_Int	channel)
 {
+	DBG( DBG_MSG , "start_scan: Hp35x0c_set_channel %d\n",channel);
 	regs[0x2f] = (regs[0x2f] & ~0xc0) | (channel << 6);
+#ifdef DEBUG
+	Hp_rts_DumpBits(0x2f, regs[0x2f]);
+#endif
 	return 0;
 }
 
@@ -699,10 +722,16 @@ Hp35x0c_set_single_channel_scanning(		SANE_Byte *regs,
 /****************************************************************************/
 					SANE_Int	on)
 {
-	if (on)
+	if (on){
 		regs[0x2f] |= 0x20;
-	else
+		DBG( DBG_MSG , "start_scan: Hp35x0c_set_single_channel_scanning on\n");
+	}else{
 		regs[0x2f] &= ~0x20;
+		DBG( DBG_MSG , "start_scan: Hp35x0c_set_single_channel_scanning off\n");
+	}
+#ifdef DEBUG
+	Hp_rts_DumpBits(0x2f, regs[0x2f]);
+#endif
 	return 0;
 }
 
@@ -712,10 +741,16 @@ Hp35x0c_set_colour_mode(			SANE_Byte *regs,
 /****************************************************************************/
 					SANE_Int	on)
 {
-	if (on)
+	if (on){
+		DBG( DBG_MSG , "start_scan: Set register to color mode\n");
 		regs[0x2f] |= 0x02;
-	else
+	}else{
+		DBG( DBG_MSG , "start_scan: Set register to gray mode\n");
 		regs[0x2f] &= ~0x02;
+	}
+#ifdef DEBUG
+	Hp_rts_DumpBits(0x2f, regs[0x2f]);
+#endif
 	return 0;
 }
 
@@ -1108,8 +1143,7 @@ Hp35x0c_rewind(THWParams *pHWParams)
 
 	/* fast move */
 	regs[0x39] = 15;
-	/* Motor enable: Muss bit 7 setzen. lscht aber bit 0-2 und
-		 setzt dann bit 0
+	/* Motor enable: It set bit 7. clear bit 0-2 and set bit 0
 		regs[0xc3] = (regs[0xc3] & 0xf8) | 1; */
 	regs[0xc3] |= 0x80;
 
@@ -1203,12 +1237,18 @@ SANE_Bool Hp35x0c_init_scan(THWParams *pHWParams, TScanParams *pParams,
 
 	for (ires = 0; Hp35x0c_resparms[ires].resolution &&
 								 Hp35x0c_resparms[ires].resolution != pParams->iDpi; ++ires);
-	if (Hp35x0c_resparms[ires].resolution == 0)
+	if (Hp35x0c_resparms[ires].resolution == 0){
+		DBG(DBG_MSG,"Hp35x0c_init_scan: did not found this resolution %d\n",pParams->iDpi);
 		return SANE_FALSE;
+	}
 
-	DBG(DBG_MSG,"start_scan: Use ires entry %d (%ddpi) for the resolution %d dpi\n",ires,
+	DBG(DBG_MSG,"Hp35x0c_init_scan: Use ires entry %d (%ddpi) for the resolution %d dpi\n",ires,
 			Hp35x0c_resparms[ires].resolution, pParams->iDpi);
 
+#ifdef ENABLE_VI8920
+	DBG(DBG_MSG,"Vi8920 is enabled and will be run us a HP3500c\n");
+#endif
+
 	/* Initialise and power up */
 #ifdef DEBUG_HP3500
 	Hp35x0c_set_all_registers(iHandle,Hp44x0_switch_on_regs);
@@ -1275,10 +1315,27 @@ SANE_Bool Hp35x0c_init_scan(THWParams *pHWParams, TScanParams *pParams,
 				0xc6, 0xc6, 0x0b);
 	pHWParams->regs[0x0b] = 0x70;/* If set to 0x71, the alternative, all values are low*/
 
-	Hp35x0c_set_channel(pHWParams->regs, RT_CHANNEL_ALL);
-	Hp35x0c_set_single_channel_scanning(pHWParams->regs, 0);
-	Hp35x0c_set_merge_channels(pHWParams->regs, 0);
-	Hp35x0c_set_colour_mode(pHWParams->regs, 1);
+	switch (pParams->mode) {
+		case GRAY:
+			DBG(DBG_ERR, "Hp35x0c_init_scan: Use gray parameter\n");
+			Hp35x0c_set_channel(pHWParams->regs, RT_CHANNEL_RED);
+			Hp35x0c_set_single_channel_scanning(pHWParams->regs, 1); /* 0x20 on / 0ff*/
+			Hp35x0c_set_merge_channels(pHWParams->regs, 1); /* 0x14 on / off */
+			Hp35x0c_set_colour_mode(pHWParams->regs, 0); /* 0x02 on / off */
+			break;
+		case COLOR:
+			DBG(DBG_ERR, "Hp35x0c_init_scan: use color parameter\n");
+			Hp35x0c_set_channel(pHWParams->regs, RT_CHANNEL_ALL);
+			Hp35x0c_set_single_channel_scanning(pHWParams->regs, 0); /* 0x20 on / 0ff*/
+			Hp35x0c_set_merge_channels(pHWParams->regs, 1); /* 0x14 on / off */
+			/*	Hp35x0c_set_merge_channels(pHWParams->regs, 0); * 0x14 on / off */
+			Hp35x0c_set_colour_mode(pHWParams->regs, 1); /* 0x02 on / off */
+			break;
+
+		default:
+			DBG(DBG_ERR, "Hp35x0c_init_scan: Invalid parameter\n");
+			return SANE_FALSE;
+	}/* switch */
 
 	Hp35x0c_set_motor_movement_clock_multiplier(pHWParams->regs,
 		Hp35x0c_resparms[ires].motor_movement_clock_multiplier);
@@ -1304,42 +1361,38 @@ SANE_Bool Hp35x0c_init_scan(THWParams *pHWParams, TScanParams *pParams,
 	x2 = pParams->iWidth + pParams->iX;
 	y = pParams->iY;
 	h = pParams->iLenght;/* * 2;*/
-	DBG(DBG_MSG,"start_scan: calculate scandata x1 = %d; x2 = %d; lenght = %d\n",x1,x2,h);
+	DBG(DBG_MSG,"Hp35x0c_init_scan: calculate scandata x1 = %d; x2 = %d; lenght = %d\n",x1,x2,h);
 	pDataPipe->iScanned = h;
 	DBG(DBG_MSG,"Hp35x0c_init_scan: Hp35x0c_set_horizontal_resolution %d.\n",pParams->iDpi);
 	Hp35x0c_set_horizontal_resolution(pHWParams->regs, pParams->iDpi);
 
 	/* modify reg 60 + 61 */
 /*	Hp_rts_set_noscan_distance(pHWParams->regs, y *
-			 Hp35x0c_resparms[ires].scan_frequency -  1);
-	pHWParams->regs[0x60] = 0x01;
-	pHWParams->regs[0x61] = 0x00;*/
+			 Hp35x0c_resparms[ires].scan_frequency -  1);*/
 	Hp_rts_set_noscan_distance(pHWParams->regs, 0x01 );
 
 	/* modify reg 62 + 63 */
 /*	Hp_rts_set_total_distance(pHWParams->regs, Hp35x0c_resparms[ires].scan_frequency *
 				(y + h + ((pParams->mode == COLOR) ? (Hp35x0c_resparms[ires].red_green_offset +
 				Hp35x0c_resparms[ires].green_blue_offset) : 0) +
-				Hp35x0c_resparms[ires].intra_channel_offset));
-	pHWParams->regs[0x62]=(pParams->iLenght + 1) & 0xff;
-	pHWParams->regs[0x63]=((pParams->iLenght + 1) >> 8) & 0xff;*/
+				Hp35x0c_resparms[ires].intra_channel_offset));*/
 	Hp_rts_set_total_distance(pHWParams->regs, h );
 
 	/* modify reg 66 + 67 */
-/*	Hp_rts_set_scanline_start(pHWParams->regs, x1 * (1200 / pParams->iDpi) / (Hp35x0c_resparms[ires].cph0s ? 1 : 2));
-	pHWParams->regs[0x66]= x1 & 0xff;
-	pHWParams->regs[0x67]= (x1 >> 8) & 0xff;*/
+/*	Hp_rts_set_scanline_start(pHWParams->regs, x1 * (1200 / pParams->iDpi) / (Hp35x0c_resparms[ires].cph0s ? 1 : 2));*/
 	Hp_rts_set_scanline_start(pHWParams->regs, x1 );
 
 	/* modify reg 6C + 6D */
-/*	Hp_rts_set_scanline_end(pHWParams->regs, (x1 + x2) * (1200 / pParams->iDpi) / (Hp35x0c_resparms[ires].cph0s ? 1 : 2));
-	pHWParams->regs[0x6c]= x2 & 0xff;
-	pHWParams->regs[0x6d]= (x2 >> 8) & 0xff;*/
+/*	Hp_rts_set_scanline_end(pHWParams->regs, (x1 + x2) * (1200 / pParams->iDpi) / (Hp35x0c_resparms[ires].cph0s ? 1 : 2));*/
 	Hp_rts_set_scanline_end(pHWParams->regs, x2 );
 
 #if 1
 	DBG(DBG_MSG,"Hp35x0c_init_scan: Hp35x0c_set_all_registers\n");
-	DumpHex(pHWParams->regs,255,16,SANE_TRUE);
+	Hp_rts_DumpHex(pHWParams->regs,255,16,SANE_TRUE);
+	Hp_rts_DumpBits(0x2f, pHWParams->regs[0x2f]);
+#endif
+#ifdef DEBUG_HP3500
+	return SANE_FALSE;
 #endif
 	Hp35x0c_set_all_registers(iHandle, pHWParams->regs);
 
@@ -1348,7 +1401,7 @@ SANE_Bool Hp35x0c_init_scan(THWParams *pHWParams, TScanParams *pParams,
 	Hp_rts_start_moving(iHandle);
 	usleep(10000);
 	/* wait for moving and check it */
-	return(Hp_rts_Check_Scanning (iHandle));
+	return(Hp_rts_Check_Moving (iHandle));
 }
 
 /****************************************************************************/
